﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2017 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System;
using System.Collections.Generic;
using HeuristicLab.Core;
using HeuristicLab.Optimization;
using HeuristicLab.Problems.DataAnalysis;

namespace HeuristicLab.Algorithms.DataAnalysis {
  internal class M5CreationParameters {
    private readonly ISplitType impurity;
    private readonly IPruningType pruningType;
    private readonly ILeafType<IRegressionModel> leafType;
    private readonly int minLeadSize;
    private readonly IRegressionProblemData problemData;
    private readonly IRandom random;
    private readonly ResultCollection results;
    public ISplitType Split {
      get { return impurity; }
    }
    public IPruningType Pruningtype {
      get { return pruningType; }
    }
    public ILeafType<IRegressionModel> LeafType {
      get { return leafType; }
    }
    public int MinLeafSize {
      get { return minLeadSize; }
    }
    private IRegressionProblemData ProblemData {
      get { return problemData; }
    }
    public IRandom Random {
      get { return random; }
    }
    public ResultCollection Results {
      get { return results; }
    }

    public ILeafType<IRegressionModel> PruningLeaf {
      get { return Pruningtype.ModelType(LeafType); }
    }
    public IEnumerable<string> AllowedInputVariables {
      get { return ProblemData.AllowedInputVariables; }
    }
    public string TargetVariable {
      get { return ProblemData.TargetVariable; }
    }
    public IDataset Data {
      get { return ProblemData.Dataset; }
    }

    public M5CreationParameters(IPruningType pruning, int minleafSize, ILeafType<IRegressionModel> modeltype,
      IRegressionProblemData problemData, IRandom random, ISplitType split, ResultCollection results) {
      impurity = split;
      pruningType = pruning;
      this.problemData = problemData;
      this.random = random;
      leafType = modeltype;
      this.results = results;
      var pruningLeaf = pruning.ModelType(LeafType);
      minLeadSize = Math.Max(pruningLeaf == null ? 0 : pruningLeaf.MinLeafSize(problemData), Math.Max(minleafSize, modeltype.MinLeafSize(problemData)));
    }
  }
}